
import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;

public class Create_Luminance_Channel implements PlugInFilter {

	ImageStack stack;
	public int setup(String arg, ImagePlus imp) { 
		stack = imp.getStack(); 
		return DOES_32 + STACK_REQUIRED; 
	}

public void run(ImageProcessor ip) {

int w =  stack.getWidth();
int h = stack.getHeight();
int nPoints =w*h;
int d = stack.getSize();

String[] sliceNames = new String[d];
sliceNames = stack.getSliceLabels();
boolean[] sel = new boolean[d];
int nSel = 0;

	GenericDialog gd = new GenericDialog("Create Luminance Channel");
			gd.addMessage("Select which channels should be used to create the\nluminance channel (they will be averaged equally)");
			for(int i=0; i<d; i++)
				gd.addCheckbox(sliceNames[i], false);

	gd.showDialog();
	if (gd.wasCanceled())
		return;

IJ.log("_____________________________________________");
IJ.log("------------Creating Luminance Channel------------");
IJ.log("Luminance channel created by averaging:");
for(int i=0; i<d; i++){
	sel[i] = gd.getNextBoolean();
	if(sel[i] == true){
		nSel++;
		IJ.log(sliceNames[i]);
	}
}

if(nSel == 0){
	IJ.log("Error - no channels selected");
	return;
}

//IJ.log("nSel: " + nSel);

//------------Import Pixel values---------------

float[] tPxs = new float[nPoints]; // temp array for loading
float[] lum = new float[nPoints];
double[] lumd = new double[nPoints];

for(int i=0; i<d; i++)
if(sel[i] == true){
	tPxs = (float[]) stack.getPixels(i+1);
	for(int k=0; k<nPoints; k++)
		lum[k] = (float) (lum[k] + (tPxs[k]/ nSel));
}

stack.addSlice("lum",lum); // adds luminance slice to end of stack
}

}
